#include "point_types.h"
#include <stdio.h>
#include <stdlib.h>

bool compare_point(const Point &p1, const Point &p2);
bool compare_pair(const PointPair &pair1, const PointPair &pair2);

void write_pairs_to_binary(const PairArray &arr, const char *filename) {
  FILE *fp = fopen(filename, "w");

  if (fp) {
    fwrite(&arr, sizeof(arr.count), 1, fp);

    u64 total_size = sizeof(*(arr.pairs)) * arr.count;

    fwrite(arr.pairs, total_size, 1, fp);

    fclose(fp);
  }
}

void read_pairs_from_binary(PairArray &arr, const char *filename) {
  FILE *fp = fopen(filename, "r");

  if (fp) {
    fread(&(arr.count), sizeof(arr.count), 1, fp);

    arr.pairs = (PointPair *)malloc(arr.count * sizeof(PointPair));

    fread(arr.pairs, sizeof(PointPair), arr.count, fp);

    fclose(fp);
  }
}

void write_pairs_to_json(const PairArray &arr, const char *filename) {
  FILE *fp = fopen(filename, "w");

  if (fp) {
    fprintf(fp, "{\n\t\"pairs\": [\n");

    for (u64 i = 0; i < arr.count; ++i) {
      PointPair pair = arr.pairs[i];
      fprintf(fp,
              "\t\t{\"x0\": %.16f, \"y0\": %.16f, \"x1\": %.16f, \"y1\": "
              "%.16f}%s\n",
              pair.p1.x, pair.p1.y, pair.p2.x, pair.p2.y,
              i + 1 < arr.count ? "," : "");
    }

    fprintf(fp, "\t]\n}\n");

    fclose(fp);
  }
}

bool compare_pair_array(const PairArray &arr1, const PairArray &arr2) {
  if (arr1.count != arr2.count) {
    return false;
  }

  for (u64 i = 0; i < arr1.count; ++i) {
    if (!compare_pair(arr1.pairs[i], arr2.pairs[i])) {
      return false;
    }
  }

  return true;
}

bool compare_point(const Point &p1, const Point &p2) {
  return p1.x == p2.x && p1.y == p2.y;
}

bool compare_pair(const PointPair &pair1, const PointPair &pair2) {
  return compare_point(pair1.p1, pair2.p1) && compare_point(pair1.p2, pair2.p2);
}
